"""
Module contains pydantic model for events meta
"""
import re

from pydantic import Extra, root_validator

from ..db.user_defined_filters import META_OPERATOR_MAP
from ..web.validators import validateMetaFilter
from .bases import JsonBaseModel

META_RX = re.compile(r"^(?P<path>(\w+[.])*([a-zA-Z0-9]+[_]?)+)" rf"(__(?P<operator>({'|'.join(META_OPERATOR_MAP)})))?$")


class Meta(JsonBaseModel):
    """
    Meta validation class
    """

    class Config:
        """Pydantic model config."""

        extra = Extra.allow

    @root_validator(pre=False, skip_on_failure=True)
    def mainValidator(cls, values):  # pylint: disable-msg=no-self-argument
        """
        Main validator checks:
            - meta validation
        Args:
            values: loaded values
        Returns:
            loaded values
        """
        if values:
            try:
                for rawname, rawvalue in values.items():
                    parsed = META_RX.match(rawname)
                    if not parsed:
                        raise ValueError("Wrong filter name")
                    validateMetaFilter(rawvalue, parsed.group("operator") or "eq")
            except Exception as exc:
                raise ValueError(f"Validation of filter '{rawname}' has failed. {exc.args[0]}") from exc
        return values

    def asDict(self) -> dict:
        """
        Overrides super.asDict to prevent using alias for meta filters keys
        """
        return self.__dict__
